/*---------------------------------------------------------------------------*/
/*   BOP   */
/*
   *************   INCLUDE:   mydefs.h   ***************
*/

/*  DESCRIPTION:
     general purpose definitions for GeoFEST finite element program
*/
/*   EOP   */
/*---------------------------------------------------------------------------*/
/*
***                           File mydefs.h                       ***
***                        GeoFEST version 4.5
*** Copyright (c) 2003, California Institute of Technology        ***
*** U.S.Sponsorship under NASA Contract NAS7-1260 is acknowledged ***
***
*** This software is designated for public release under JPL Task ***
*** Plan Number 83-6791 and may be publicly released after        ***
*** approval for release has been granted by GSFC Software        ***
*** Release  Authority
***
***/
/*
***GeoFEST version 4.5***
G. A. Lyzenga -- unified parallel/serial version and bug fixes 11/17/03
G. A. Lyzenga -- split node enhancements 8/16/02
G. A. Lyzenga -- modification to correctly relax hydrostatic stresses 6/15/00
J. W. Parker -- 3D support and deleted hypercube branches 9/16/98
G. A. Lyzenga  -- last modified on 3/03/95
Modification history:
2.1: fixes bug in boundary velocities and changes
     the way stresses are output.
2.2: implements split node faults
2.3: adds pass-thru comments
3.0: 3-D tetrahedra supported, hypercube branches deleted
3.1: hydrostatic relaxation changes
4.0: name change from visco to GeoFEST; correct implementation of 2-D out-of-
     plane stresses as well as hydrostatic relaxation; culling of remaining
     vestiges of hypercube stuff; one code without recompilation for both
     2-D and 3-D domains
4.1: Introduction of trilinear hexahedral bricks and PCG iterative solver option
4.2: Bug fixes; standard source prologues added (June 02)
4.3: Corrects inefficiency in split node algorithm and introduces new
     split node input format
4.4: Adds dummy calls to MPI and Pyramid to provide a single unified
     source for parallel and serial codes.  Also introduces a major
     algorithmic enhancement; stress equilibrium maintenance during visco
     elastic stepping for long-term stability.  Also more minor bug fixes.
4.5: Supports surface tractions and roller/spring duo options for tet elements.
     Provides enhanced conjugate gradient convergence monitoring.
 $Log: mydefs.h,v $
 Revision 1.3  2003/02/09 06:10:56  lyzenga
 *** empty log message ***

     split node input format
*/

/* general purpose definitions */
#define TRUE 1
#define FALSE 0
#define MAXSUB  (pow(2.0,20.0))
#define FOREVER    for (;;)
#define FAIL       1
#define SUCCEED    0
#define YES     1
#define NO      0
#define ABS(x)     (((x) < 0) ? -(x) : (x))
#define MAX(x,y)   (((x) < (y)) ? (y) : (x))
#define MIN(x,y)   (((x) < (y)) ? (x) : (y))
#define FSIGNUM(x)   (((x) >= 0.0) ? (1.0) : (-1.0))

#define real double
#define HZ 100

#define SET_TRUE(x)   x=1
#define SET_FALSE(x)  x=0
#define BETWEEN(x,y,z) (((x) <= (y) && (y) <= (z)) ? 1 : 0)
#define COUNTER    int
#define ZERO       0.0
#define PT25       0.25
#define PT5        0.5
#define ONE        1.0
#define TWO        2.0
#define THREE      3.0
#define FOUR       4.0
#define FIVE       5.0
#define SIX        6.0
