#ifndef	GUARD_OUTPUT_PHASE_H
#define	GUARD_OUTPUT_PHASE_H 1

#include <stdio.h>
#include <stdlib.h>

#define INLINE inline
#define	SPATIAL_DIMENSIONS 3
#define	DEGREES_OF_FREEDOM 3

typedef	struct Coordinates {
  double element[SPATIAL_DIMENSIONS];
  } Coordinates;

typedef	struct Displacement {
  double element[DEGREES_OF_FREEDOM];
  } Displacement;

typedef	struct deltaDisplacement {
  double element[DEGREES_OF_FREEDOM];
  } deltaDisplacement;

typedef struct out_node_data {
  size_t		node;
  size_t		group;
  Coordinates		coords;
  Displacement		displ;
  deltaDisplacement	del_displ;
  } OUT_NODE_DATA;

INLINE static
void out_node_data_init(
    OUT_NODE_DATA*	p,
    size_t		node,
    size_t		group,
    const double	coords[],
    const double	displ[],
    const double	del_displ[]
    ) {
  size_t  i ;
  
  p->node	= node;
  p->group	= group;
  for(i=0;i<SPATIAL_DIMENSIONS;i++)
     {
      if(i >= fe_sys.nsd)  p->coords.element[i]=0.0 ;
      else  p->coords.element[i]=coords[i] ;
     }
  for(i=0;i<DEGREES_OF_FREEDOM;i++)
     {
      if(i >= fe_sys.ndof)  {p->displ.element[i]=0.0 ; p->del_displ.element[i]=0.0 ;}
      else  {p->displ.element[i]=displ[i] ; p->del_displ.element[i]=del_displ[i] ;}
     }
  }

INLINE static
void out_node_data_destroy(
    const
    OUT_NODE_DATA*	p
    ) {
  }

INLINE static
void out_node_data_destroy_list(
    const
    OUT_NODE_DATA	p[],
    size_t              records
    ) {
  }

INLINE static
int out_node_data_fprintf(
    FILE*	stream,
    const	OUT_NODE_DATA* p
  ) {
  int characters = fprintf(stream, "node %d", p->node);
  { const
    size_t	n = fe_sys.nsd;
    size_t	j = 0;
    for (j = 0; j < n; ++j)
      characters += fprintf(stream, "   %f", p->coords.element[j]);
    }
  { const
    size_t	n = fe_sys.ndof;
    size_t	j = 0;
    for (j = 0; j < n; ++j)
      characters += fprintf(stream, "   %g", p->displ.element[j]);
    }
  { const
    size_t	n = fe_sys.ndof;
    size_t	j = 0;
    for (j = 0; j < n; ++j)
      characters += fprintf(stream, "   %g", p->del_displ.element[j]);
    }
  characters += fprintf(stream, "\n");
  return characters;
  }

INLINE static
int out_node_data_compar(const void* p, const void* q) {
  return (*((const OUT_NODE_DATA**)p))->node
       - (*((const OUT_NODE_DATA**)q))->node;
  }

INLINE static const
OUT_NODE_DATA** out_node_data_sort(
    /* This function allocates and returns an array of pointers
     * which must be free'd by the calling program.			*/
    const
    OUT_NODE_DATA	q[],
    size_t		records
    ) {
  const
  OUT_NODE_DATA**	p = calloc(records, sizeof(OUT_NODE_DATA*));
  size_t		record = 0;
  for (record = 0; record < records; ++record)
    p[record] = &q[record];
  qsort((void*)p, records, sizeof(OUT_NODE_DATA*), out_node_data_compar);
  return p;
  }

INLINE static
int out_node_data_fprintf_list(
    FILE*		stream,
    const
    OUT_NODE_DATA	q[],
    size_t		records
    ) {
  const
  OUT_NODE_DATA**	p = out_node_data_sort(q, records);
  int			characters = 0;
  size_t		record = 0;
  for (record = 0; record < records; ++record)
    characters += out_node_data_fprintf(stream, p[record]);
  free((void*)p);
  return characters;
  }

typedef	struct Stress {
  double element[2*DEGREES_OF_FREEDOM];
  } Stress;

typedef struct out_element_data {
  size_t		element;
  size_t		group;
  Coordinates		coords;
  Stress		stress;
  } OUT_ELEMENT_DATA;

INLINE static
void out_element_data_init(
    OUT_ELEMENT_DATA*	p,
    size_t		element,
    size_t		group,
    size_t      str_size,
    const double	coords[],
    const double	stress[]
    ) {
  size_t  i ;

  p->element	= element;
  p->group	= group;
  for(i=0;i<SPATIAL_DIMENSIONS;i++)
     {
      if(i >= fe_sys.nsd)  p->coords.element[i]=0.0 ;
      else  p->coords.element[i]=coords[i] ;
     }
  for(i=0;i<2*DEGREES_OF_FREEDOM;i++)
     {
      if(i >= str_size)  p->stress.element[i]=0.0 ;
      else  p->stress.element[i]=stress[i] ;
     }
  }

INLINE static
void out_element_data_destroy(
    const
    OUT_ELEMENT_DATA*	p
    ) {
  }

INLINE static
void out_element_data_destroy_list(
    const
    OUT_ELEMENT_DATA	p[],
    size_t              records
    ) {
  }

INLINE static
int out_element_data_fprintf(
    FILE*	stream,
    const	OUT_ELEMENT_DATA* p,
    size_t  str_size
  ) {
  int characters = 0;
  { const
    size_t	n = fe_sys.nsd;
    size_t	j = 0;
    for (j = 0; j < n; ++j)
      characters += fprintf(stream, "   %f", p->coords.element[j]);
    }
  { const
    size_t	n = str_size;
    size_t	j = 0;
    for (j = 0; j < n; ++j)
      characters += fprintf(stream, "   %f", p->stress.element[j]);
    }
  characters += fprintf(stream, "   %d\n", p->element);
  return characters;
  }

INLINE static
int out_element_data_compar(const void* p, const void* q) {
  /* This routine must be modified to sort on group number.		*/
  return (*((const OUT_ELEMENT_DATA**)p))->element
       - (*((const OUT_ELEMENT_DATA**)q))->element;
  }

INLINE static const
OUT_ELEMENT_DATA** out_element_data_sort(
    /* This function allocates and returns an array of pointers
     * which must be free'd by the calling program.			*/
    const
    OUT_ELEMENT_DATA	q[],
    size_t		records
    ) {
  const
  OUT_ELEMENT_DATA**	p = calloc(records, sizeof(OUT_ELEMENT_DATA*));
  size_t		record = 0;
  for (record = 0; record < records; ++record)
    p[record] = &q[record];
  qsort((void*)p, records, sizeof(OUT_ELEMENT_DATA*), out_element_data_compar);
  return p;
  }

INLINE static
int out_element_data_fprintf_list(
    FILE*		stream,
    const
    OUT_ELEMENT_DATA	q[],
    size_t		records,
    size_t      str_size
    ) {
  const
  OUT_ELEMENT_DATA**	p = out_element_data_sort(q, records);
  int			characters = 0;
  size_t		record = 0;
  for (record = 0; record < records; ++record)
    characters += out_element_data_fprintf(stream, p[record], str_size);
  free((void*)p);
  return characters;
  }

#undef	DEGREES_OF_FREEDOM
#undef	SPATIAL_DIMENSIONS
#undef	INLINE
#endif/*GUARD_OUTPUT_PHASE_H 1	*/

